/**************************************************************** Pulse.c
Blinks one output in one of 7 patterns at a variable frequency

Written for the 12F675 chip
Compiles with CC5X compiler

Inputs:  2 Pushbuttons/signals to ground (mode, frequency)
Outputs: 1 LED

... theCoder, Sept '07
****************************************************************************/

#pragma config |= 0x33C4

typedef uns8 byte;
typedef uns16 word;

#define SET 1
#define RESET 0

// eeprom storage locations
#define MODE_ADDRESS 0xd0
#define PULSEFREQ_ADDRESS 0xd1

// pulse limits
#define PULSEFREQ_MAX 150
#define PULSEFREQ_MIN 0	

// I/O pins
#define PB_MODE GPIO.0  		// mode button (pin 7 on 12F675)
#define PB_FREQUENCY GPIO.1		// frequency button (pin 6 on 12F675)
#define LED_1 GPIO.5			// LED output (pin 2 on 12F675)



byte state;						// state machine state 
word timer;						// loop counter used in state machine
byte dim;						// ratio of on to off in PWM logic
long ondur;						// how long to have LED on in PWM logic
long offdur;					// how long to have LED off in PWM logic
byte offset;					//
byte mode;						// blink mode - how should it blink?  User selectable with PB
byte direction;					// either 1 or -1, used in smooth blink mode to count up/down
byte pulsefreq;					// blink cycle time.  User selectable with PB  
byte x;							// scratch variable

enum States		// state machine states
{
	state0,
	state1,
};

enum Modes		// mode list
{
	sawtoothup,		// gradual up, drop to off
	sawtoothdown,	// gradual down, jumps to all on
	smooth,			// gradual up, gradual down
	blink,			// 50% on, 50% off
	blip_off,		// quick on, variable off
	blip_on,		// quick off, variable on
	all_on,			// all on, variable brightness
	all_off			// off
};



//*************************************** LedOn()
void LedOn()
{
	LED_1 = SET;
}

//*************************************** LedOff()
void LedOff()
{
	LED_1 = RESET;
}

//*************************************** ReadByteFromEEProm()
// read from eeprom
byte ReadByteFromEEProm(byte add)
{
    EEADR = add;
    RD = SET;
    return EEDATA;
}

//*************************************** WriteByteToEEProm()
void WriteByteToEEProm(byte add, byte value)
// write to eeprom
{
    EEADR = add;
    EEDATA = value;
    WREN = SET;
    EECON2 = 0x55;
    EECON2 = 0xaa;
    WR = SET;
    WREN = RESET;
    while(WR);
}

//*************************************** GetMode()
// reads mode variable from eeprom	
byte GetMode()
{
	byte tmode;
 	tmode = ReadByteFromEEProm(MODE_ADDRESS); 	// read byte stored in eeprom 
	// deal with first run case and/or read/write error
	if ((tmode < sawtoothup) || (tmode > all_on)) tmode = sawtoothup;
	return tmode;
}

//*************************************** GetPulseFreq()
// reads pulse frequency variable from eeprom
byte GetPulseFreq()
{
	byte tfreq;
    tfreq = ReadByteFromEEProm(PULSEFREQ_ADDRESS);  // read byte stored in eeprom
	// deal with first run case and/or read/write error
	if (tfreq < PULSEFREQ_MIN) tfreq = PULSEFREQ_MAX - PULSEFREQ_MIN;
	if (tfreq > PULSEFREQ_MAX) tfreq = PULSEFREQ_MAX - PULSEFREQ_MIN;
	return tfreq;
}

//*************************************** Initialize()
void Initialize()
{
	// run once from main()
	clearRAM(); // clears all ram

	// initialize TMR0
	OPTION_REG = 0b00000001;
	TMR0 = 0x00;		// timer 0

	// enable digital IO
	ANSEL = 0x00; 		// turn off ADC
	CMCON = 0x07; 		// turn off comparator
	
	GPIO = 0b00000000;
	TRISIO = 0b00011;  	// set up outputs 0 & 1 as input

	mode = GetMode();
    pulsefreq = GetPulseFreq();
	direction = 1;    	// up/down variable for smooth state
	state = state0;
	timer = 0;
}


//*************************************** StateMachine()
void StateMachine()
{
	++offset;
	if (offset > pulsefreq)			// main frequency throttle
	{
		offset = 0;
		// determine PWM on/off duty cycle variables
		switch (mode)
		{
			case sawtoothdown:		// ************* sawtoothdown
				// goes instant on, tapers down
				++dim;
				if (dim > pulsefreq) 
				{
					dim = 0;
				}
				ondur = dim;
				offdur = (long) pulsefreq - dim;
			break;

			case sawtoothup:		// ************* sawtoothup
				// goes instant off, tapers up
				--dim;
				if (dim <= 5) 
				{
					dim = pulsefreq;
				}
				ondur = dim;
				offdur = (long) pulsefreq-dim;
			break;
			
			case smooth:			// ************* smooth
				// tapers up, then tapers down
				dim = dim + direction;
				if (dim <= 0 || dim >= pulsefreq) 
				{
					direction = direction * -1;
				}
				ondur = dim;
				offdur = (long) pulsefreq - dim;
			break;

			case blink:				// ************* blink
				// 50% on, 50% off 
				ondur = pulsefreq * 150;
				offdur = (long) ondur;
			break;

			case blip_off:			// ************* blip_off
				// short on, variable off
				offdur =100;
				ondur = (long) (pulsefreq * 150);
			break;

			case blip_on:			// ************* blip_off
				// short off, variable on
				offdur= (long) (pulsefreq * 150);
				ondur= 400;
			break;

			case all_on:			// ************* all_on
				// variable brightness, always on
				ondur = PULSEFREQ_MAX-pulsefreq;
				offdur = pulsefreq;
				if (ondur < 11) ondur = 0;
				if (offdur < 11) ondur = PULSEFREQ_MAX;
			break;

			case all_off:			// ************* all_off
				// off
				ondur = 250;
				offdur = 0;
			break;

		}
	} 
 
	// now to actually turn led on / off 
	if (mode == all_off) {  // cheezy hack.  Could not get it to work using ondur & offdur variables
		LedOff();
	}
	else {
		switch (state)			// simulated PWM logic
		{
			case state0:		// on state
   	         ++timer;
				if (timer > ondur)
				{
					timer = 0;
					LedOn();
					state = state1;
				}
				else LedOff();
			break;
			case state1:		// off state
				++timer;
				if (timer > (offdur))
				{
					timer = 0;
					LedOff();
					state = state0;
				}
				else LedOn();
			break;
		}	// end of switch
	}  // end of all_off IF
}


//*************************************** Main()
void main()
{
	word debounce_ct;		// loop counter to keep pushbuttons from "Pressing" hundreds of times
	debounce_ct = 0;
	byte debouncing = 0;	// are we debouncing now?

	Initialize();			// initialize chip and variables


	while (1)				// main loop
	{
		if (T0IF) 			// if timer interupt
		{
			T0IF = 0;		// reset timer
			//TMR0 = 0x08; 	// subtract 2 cycles for setting TMR0 and one for resetting flag
			TMR0 = 0xd8; 	// preset timer to get a another interupt very soon - sets timer preset
			StateMachine(); // turn LED on/off
			
			//  Change mode
			if ((PB_MODE == 0) && (!debouncing))
			{
				debouncing = 1;
				++mode;
				if (mode > all_off)		// loop around if hit max
				{
					mode =  sawtoothup;
				}
				state=state0;
				offset=0;
				WriteByteToEEProm(MODE_ADDRESS, mode);	// save value off for next power up
			}
			else	// give the guys finger a chance to get off the button
			{
				++debounce_ct;
				if (debounce_ct > 9000)
				{
					debouncing = 0;
					debounce_ct = 0;
				}
			}	
			
			// Change pulsefrequency
			if ((PB_FREQUENCY == 0) && (!debouncing))
			{
				debouncing = 1;
				pulsefreq += 5;
				if (pulsefreq > PULSEFREQ_MAX)
				{
					pulsefreq = 0;
				}
				WriteByteToEEProm(PULSEFREQ_ADDRESS, pulsefreq);
			}
			else
			{
				++debounce_ct;
				if (debounce_ct > 9000)
				{
					debouncing = 0;
					debounce_ct = 0;
				}
			}
	
		}  // end of timer interupt IF
	}  // end of main while loop
}  // end of main

